---
jupytext:
  formats: ipynb,md:myst
  text_representation:
    extension: .md
    format_name: myst
    format_version: 0.13
    jupytext_version: 1.14.1
kernelspec:
  display_name: Python 3 (ipykernel)
  language: python
  name: python3
---

# EDA Schuhgröße

Dieses Notebook: Erste Schritte für eine explorative Datenanalyse zum Datensatz "Schuhgröße", WS 2022.

+++

## Busines Case

Ursprünglich Busines Case 3 in <http://jbusse.de/2021_ws_dsci/dsci-101-statur.html#statur-bc1> , im WS 2022 aber ohne die Spalte "Gewicht".

Vorherzusagen (Target) ist die Spalte Gender. Wir nehmen an, dass in einer Konsumenten-Analyse das Attribut "Gender" nicht erhoben wird, wir aber genau daran sehr interessiert sind. Also erschließen wir Gender aus Körpergröße und Schuhgröße.

+++

## Datensatz beschreiben

Hier etwas nachforschen: Was wissen wir über die Herkunft der Daten? Die beschriebene Domäne, hier: die Menschen? Warum wurden die Daten erhoben, wofür sind sie (nicht) repräsentativ? etc.

Auch schon hier Erebnisse aus der nachfolgenden Analyse "spoilern"? Ja, denn CRISP-DM ist ein iterativer Prozess, und unser Notebook ist das Ergebnis dieses iterativen Prozesses.

+++

## Daten und Bibliotheken einlesen

```{code-cell} ipython3
import numpy as np
import pandas as pd
import seaborn as sns
from IPython.display import display, Markdown, Latex
```

```{code-cell} ipython3
random_state=42

Erwachsen_Alter = 15 # Faustregel, Klischee, gesellschaftliche Konstruktion: muss man sicherlich noch diskutieren!
```

## Datensatz als Aufgabe herstellen

```{code-cell} ipython3
dbsrc = {} # database, source version
```

```{code-cell} ipython3
#X = pd.read_csv("../data/schuhgroesse_ws2021.csv", delimiter=',')
X = pd.read_csv("../data/schuhgroesse_ws2022.csv", delimiter=',')
print(X.columns)
y = X.pop("Gender")
X.describe()
```

```{code-cell} ipython3
X.head()
```

<https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.train_test_split.html>

```{code-cell} ipython3
from sklearn.model_selection import train_test_split

X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.33, random_state=random_state)

dbsrc["train"] = X_train
dbsrc["train"]["Gender"] = y_train  # y-Spalte wieder hinzufügen

dbsrc["test"]  = X_test
dbsrc["test"]["Gender"] = y_test
```

```{code-cell} ipython3
#for z in dbsrc:
#    display(dbsrc[z].describe())
```

```{code-cell} ipython3
for z in dbsrc:
    print(f"writing file {z}.csv")
    dbsrc[z].to_csv(f"statur_{z}.csv", sep=';', index = False)
```

## Datensatz analysieren

+++

Um uns Arbeit zu ersparen, legen wir den Original- und alle Teil-Datensätze nicht als eigene Variablen, sondern als in dict von DataFrames an.

```{code-cell} ipython3
database = {} # Alle Dataframes, mit denen wir rechnen
```

```{code-cell} ipython3
database["train"] = pd.read_csv("statur_train.csv", delimiter=';')
database["train"].columns
```

```{code-cell} ipython3
assert set(database["train"].columns) == {'Körpergröße', 'Alter', 'Schuhgröße', 'Gender' }
```

## EDA incl. Cleaning

```{code-cell} ipython3
database["train"].head(3)
```

### Cleaning

```{code-cell} ipython3
database["train_clean"] = database["train"].copy() # Eine Kopie erzwingen
X = database["train_clean"] # ein Pointer, keine Kopie: Änerungen auf X ändern auch "database['train_clean']"
```

Die Gender-Spalte hat fehlerhafte Einträge:

```{code-cell} ipython3
set(database["train_clean"]["Gender"])
```

```{code-cell} ipython3
def fn_clean_gender(string):
    if string in [ None, np.nan, "" ]:
        return "NaN"
    else:
        return string[0].lower()
```

```{code-cell} ipython3
fn_clean_gender("")
database["train_clean"]["Gender"] = database["train_clean"]["Gender"].apply(lambda x: fn_clean_gender(x))
set(database["train_clean"]["Gender"])
```

### Einheit für  Körpergröße 

Das funktioniert:
```
X.Körpergröße[X.Körpergröße > 3] = X.Körpergröße/ 100 
```

Aber es kommt eine Warnung:

> SettingWithCopyWarning: A value is trying to be set on a copy of a slice from a DataFrame
>
> See the caveats in the documentation: <https://pandas.pydata.org/pandas-docs/stable/user_guide/indexing.html#returning-a-view-versus-a-copy>

Besser: "The following is the recommended access method using `.loc` for multiple items (using `mask`) "

```{code-cell} ipython3
# https://pandas.pydata.org/pandas-docs/stable/user_guide/indexing.html
mask = X.Körpergröße == 999
X.loc[mask, 'Körpergröße'] = np.nan 

mask = X.Körpergröße < 30 # kleiner als 30 cm ist niemand, also ist die Größe in m angegeben: in cm umrechnen
X.loc[mask, 'Körpergröße'] = X.loc[mask, 'Körpergröße'] * 100 
```

```{code-cell} ipython3
mask = X.Alter > 120
X.loc[mask, 'Alter'] = np.nan 
```

```{code-cell} ipython3
mask = X.Schuhgröße == 99
X.loc[mask, 'Schuhgröße'] = np.nan 
```

```{code-cell} ipython3
max(X.Alter)
```

## Datensatz visualisieren

JB: Einige Routine-Schritte sind schon gegeben, kann man aber bei Bedarf ergänzen.

```{code-cell} ipython3

# Sorge für Konsistenz in der Farbgebung von Seaborn:
# Seaborne Farben dem Geschlecht eindeutig zuweisen
markers={"Gender": ["^", "^", "^",  "o",  "o",  ".",  "."]}
palette={"Gender": {
    "f": "r", 
    "g":"r",
    "w": "r",
    "m": "b", 
    "b": "b",
    "d": "y",
    "NaN": "y",
    "n": "y"
    }}
hue="Gender"
```

Eine Vorschau auf eine lineare Regression (JB: Welche Regression verwendet Seaborne hier eigentlich?) 
 zeigt, dass das entstehende Modell suboptimal ausfallen wird, wenn man nicht nach Geschlecht oder heranwachsend/erwachsen differenziert:

```{code-cell} ipython3
sns.regplot(x="Alter", y="Schuhgröße", data=database["train_clean"])
```

Wir wollen also im Folgenden mit verschiedenen Teil-Datensätzen weiterarbeiten.

```{code-cell} ipython3
database["Erwachsen"] = database["train_clean"].copy().loc[database["train_clean"].Alter > Erwachsen_Alter]
database["Heranwachsend"] = database["train_clean"].copy().loc[database["train_clean"].Alter <= Erwachsen_Alter]
database["Frauen"] = database["train_clean"].loc[(database["train_clean"].Alter > Erwachsen_Alter) & (database["train_clean"].Gender=='f')]
database["Maenner"] = database["train_clean"].loc[(database["train_clean"].Alter > Erwachsen_Alter) & (database["train_clean"].Gender=='m')]
```

```{code-cell} ipython3
# alle Original- und Teildatensätze, für die wir die Auswertung machen wollen
focus = ["train_clean", "Heranwachsend", "Erwachsen", "Frauen", "Maenner"]
```

```{code-cell} ipython3
for f in focus:
    display(Markdown(f'**{f}**'))
    display(database[f].describe())
```

```{code-cell} ipython3
f="train_clean"
display(Markdown(f"**{f}**"))
sns.pairplot(database[f], hue=hue, 
             markers=markers[hue], palette=palette[hue]
            )
```

```{code-cell} ipython3
f="Heranwachsend"
display(Markdown(f"**{f}**"))
sns.pairplot(database[f], hue=hue, 
             # markers=markers[hue], palette=palette[hue]
            )
```

```{code-cell} ipython3
f="Erwachsen"
display(Markdown(f"**{f}**"))
sns.pairplot(database[f], hue=hue, markers=markers[hue], palette=palette[hue])
```

```{code-cell} ipython3
f = "Frauen"
display(Markdown(f"**{f}**"))
sns.pairplot(database[f], hue=hue, #markers=markers[hue], 
             palette=palette[hue])
```

```{code-cell} ipython3
# oder mehr fancy?
# https://seaborn.pydata.org/tutorial/distributions.html#plotting-many-distributions
#g = sns.PairGrid(X, hue = "Gender")
#g.map_upper(sns.histplot)
#g.map_lower(sns.kdeplot, fill=True)
#g.map_diag(sns.histplot, kde=True)
```

```{raw-cell}
noch mehr fancy: <https://www.python-graph-gallery.com/111-custom-correlogram>
```

```{raw-cell}
for f in focus:
    display(Markdown(f'**{f}**'))
    sns.jointplot(
        data=database[f],
        x="Körpergröße", y="Schuhgröße", 
        #hue=hue, 
        #markers=markers[hue], 
        palette=palette[hue],
        kind="reg",
        # kind="kde"
)
```

Da könnte man hinwollen: Heranwachsende Männer (blau), erwachsene Frauen (rot) etc.:

```{code-cell} ipython3
sns.scatterplot(x="Körpergröße", y="Schuhgröße", data=database["Heranwachsend"], color="c", hue=hue)
#sns.regplot(x="Körpergröße", y="Schuhgröße", data=database["Erwachsen"], color="c")
```

```{code-cell} ipython3

```

```{code-cell} ipython3

```
